@@ -1,7 +1,7 @@
-/* gxmessage - an xmessage clone using GTK2
+/* gxmessage - an xmessage clone using GTK
  *
  * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2012, 2014 Timothy Richard Musson
+ * 2012, 2014, 2015 Timothy Richard Musson
  *
  * Email: Tim Musson <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#gxmessage
@@ -32,7 +32,7 @@
 
 /* Details for Copyright and bug report messages: */
 #define AUTHOR  "Timothy Richard Musson"
-#define YEAR    "2014"
+#define YEAR    "2015"
 #define MAILTO  "<trmusson@gmail.com>"
 
 /* If the following file exists, the -noescape option will be allowed to work: */
@@ -401,48 +401,6 @@ cb_timeout (gint *timeout)
 }
 
 
-gint
-geometry_get_number (const gchar **geometry)
-{
-	/* (This function is taken from GNOME's gnome-geometry.c) */
-
-	gint value = 0;
-	gint mult  = 1;
-
-	if (**geometry == '-') {
-		mult = -1;
-		(*geometry)++;
-	}
-	while ( **geometry && isdigit(**geometry) ) {
-		value = value * 10 + (**geometry - '0');
-		(*geometry)++;
-	}
-	return value * mult;
-}
-
-
-void
-geometry_parse_string (const gchar *geometry, gint *width, gint *height)
-{
-	/* Parse geometry string into width and height */
-
-	*width = *height = -1;
-
-	if (*geometry == '=') geometry++;
-	if (*geometry == '\0') return;
-
-	*width = geometry_get_number (&geometry);
-	if (*width < -1) *width = -1;
-
-	if (*geometry == 'x') {
-		geometry++;
-		if (*geometry == '\0') return;
-		*height = geometry_get_number (&geometry);
-		if (*height < -1) *height = -1;
-	}
-}
-
-
 void
 window_create (void)
 {
@@ -453,7 +411,7 @@ window_create (void)
 	GtkWidget  *btn;
 	GtkWidget  *message_widget;
 	Button     *button;
-	GdkColor   color;
+	GdkRGBA    color;
 	GtkRequisition size_req;
 	GtkTextBuffer *buf;
 	GtkTextIter iter;
@@ -520,7 +478,7 @@ window_create (void)
 	if (gx.font_str != NULL) {
 		PangoFontDescription *font_desc;
 		font_desc = pango_font_description_from_string (gx.font_str);
-		gtk_widget_modify_font (message_widget, font_desc);
+		gtk_widget_override_font (message_widget, font_desc);
 		pango_font_description_free (font_desc);
 	}
 
@@ -533,13 +491,15 @@ window_create (void)
 	gtk_text_buffer_place_cursor (buf, &iter);
 
 	if (gx.color_fg != NULL) {
-		if (gdk_color_parse (gx.color_fg, &color)) {
-			gtk_widget_modify_text (message_widget, GTK_STATE_NORMAL, &color);
+		if (gdk_rgba_parse (&color, gx.color_fg)) {
+			gtk_widget_override_color (message_widget,
+			  GTK_STATE_NORMAL, &color);
 		}
 	}
 	if (gx.color_bg != NULL) {
-		if (gdk_color_parse (gx.color_bg, &color)) {
-			gtk_widget_modify_base (message_widget, GTK_STATE_NORMAL, &color);
+		if (gdk_rgba_parse (&color, gx.color_bg)) {
+			gtk_widget_override_background_color (message_widget,
+			  GTK_STATE_NORMAL, &color);
 		}
 	}
 
@@ -574,6 +534,7 @@ window_create (void)
 		while (button != NULL) {
 
 			if (strcmp (button->label, "okay") == 0) {
+				/* XXX: gtk_button_new_from_stock deprecated since GTK 3.10 */
 				btn = gtk_button_new_from_stock ("gtk-ok");
 			}
 			else if (g_str_has_prefix (button->label, "GTK_STOCK_")) {
@@ -586,6 +547,7 @@ window_create (void)
 					s++;
 				}
 				s = g_strconcat ("gtk-", p, NULL);
+				/* XXX: gtk_button_new_from_stock deprecated since GTK 3.10 */
 				btn = gtk_button_new_from_stock (s);
 				g_free (s);
 				g_free (p);
@@ -620,9 +582,18 @@ window_create (void)
 		win_h = max_h;
 	}
 	else {
+
 		GtkWidget *dummy = gtk_label_new (gx.message_text);
-		gtk_widget_modify_font (dummy,
-		    gtk_widget_get_style (message_widget)->font_desc);
+		GtkStyleContext *context;
+		PangoFontDescription *font_desc;
+
+		context = gtk_widget_get_style_context (message_widget);
+		gtk_style_context_get (context, GTK_STATE_FLAG_NORMAL,
+		  GTK_STYLE_PROPERTY_FONT, &font_desc, NULL);
+		gtk_widget_override_font (dummy, font_desc);
+
+		pango_font_description_free (font_desc);
+
 		gtk_container_add (GTK_CONTAINER(vbox), dummy);
 		gtk_widget_get_preferred_size (dummy, &size_req, NULL);
 		gtk_widget_destroy (dummy);
@@ -644,18 +615,15 @@ window_create (void)
 	}
 
 	if (gx.geom_str != NULL) {
-		gint tmp_w, tmp_h;
-		geometry_parse_string (gx.geom_str, &tmp_w, &tmp_h);
-		if (tmp_w != -1) win_w = tmp_w;
-		if (tmp_h != -1) win_h = tmp_h;
+		gtk_widget_show_all (vbox); /* must precede parse_geometry */
+		gtk_window_parse_geometry (GTK_WINDOW(window), gx.geom_str);
 	}
 
-	gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
-	gtk_window_set_position (GTK_WINDOW(window), gx.window_position);
 
 	/* open the window */
 	gtk_widget_show_all (window);
 
+
 	/* begin timeout */
 	if (gx.timeout != 0) {
 		gx.timeout_id = g_timeout_add (1000, (GSourceFunc)cb_timeout,
